
use inquire::{ui::RenderConfig, Select};

use super::*;

pub struct MainMenu {
    pub menu: Vec<Box<dyn for<'a> Flow<'a> + Send>>
}

impl MainMenu {
    pub fn new() -> Self {
        MainMenu { 
            menu: vec![]
        }
    }
}

unsafe impl Send for MainMenu {

}

#[async_trait]
impl<'a> Flow<'a> for MainMenu {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a> {
        let choice = Select::new("What would you like to do?", self.menu)
            .with_help_message("Select the action you want to take or quit the program")
            .with_render_config(RenderConfig {
                option_index_prefix: inquire::ui::IndexPrefix::Simple,
                ..Default::default()
            })
            .prompt_skippable()?;

        if let Some(choice) = choice {
            let (outcome, new_state) = choice.run(state).await?;
            
            if outcome == FlowOutcome::Abort {
                Ok((outcome, new_state))
            }
            else {
                self.run(new_state).await
            }       
        }
        else {
            return Ok((FlowOutcome::Abort, state))
        }
    }
}

impl Display for MainMenu {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Main menu")
    }
}