use std::fmt::Display;

use async_trait::async_trait;
use gamenight_api_client_rs::apis::configuration::Configuration;
use inquire::InquireError;

pub mod main;
pub mod login;
mod main_menu;
mod abort;
mod list_gamenights;

pub struct GamenightState {
    configuration: Configuration,
}

impl GamenightState {
    pub fn new() -> Self{
        Self {
            configuration: Configuration::new()
        }
    }
}

#[derive(Debug)]
pub struct FlowError {
    pub error: String
}

impl From<InquireError> for FlowError {
    fn from(value: InquireError) -> Self {
        Self {
            error: value.to_string()
        }
    }
}

impl<T> From<gamenight_api_client_rs::apis::Error<T>> for FlowError {
    fn from(value: gamenight_api_client_rs::apis::Error<T>) -> Self {
        Self {
            error: value.to_string()
        }
    }
}

#[derive(PartialEq)]
pub enum FlowOutcome {
    Successful,
    Bool(bool),
    String(String),
    Abort
}

type FlowResult<'a> = Result<(FlowOutcome, &'a mut GamenightState), FlowError>;

#[async_trait]
pub trait Flow<'a>: Sync + Display {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a>;
}